{ **************************************************************************** }
{ *                                                                          * }
{ * DONKEY KONG (Nintendo, Game & Watch Multi Screen) Simulator              * }
{ * by Luca "MADrigal" Antignano, May 2000 (revision: March 2022)            * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit unit1;
{ this unit contains the main form implementation                              }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Math, Registry, Jpeg, Pngimage, Fmod, Fmodtypes, StdCtrls, Menus,
  ShellAPI;

type
  TForm1 = class(TForm)
{ * HANDHELD GRAPHICS, INCLUDING BUTTONS                                     * }

    // fullscreen mode items
    btn_game_menu_top:          TImage;
    btn_game_menu_background:   TImage;

    im_backdrop:                TImage;
    im_backdrop_logo:           TImage;

    // electronic game
    im_background_open:         TImage;     im_background_closed:       TImage; {1-2}
    im_background_screen_1_top: TImage;     im_background_screen_2_top: TImage; {3-4}

    btn_game_a_down:            TImage;     btn_game_a_top:             TImage; {5-6}
    btn_game_b_down:            TImage;     btn_game_b_top:             TImage; {7-8}
    btn_time_down:              TImage;     btn_time_top:               TImage; {9-10}
    btn_acl_down:               TImage;     btn_acl_top:                TImage; {11-12}
                                            btn_alarm_top:              TImage; {13}

    btn_cross_down:             TImage;     btn_cross_top:              TImage; {14-15}
    btn_jump_down:              TImage;     btn_jump_top:               TImage; {16-17}

    btn_cross_down_up:          TImage;                                         {134}
    btn_cross_down_down:        TImage;                                         {135}
    btn_cross_down_left:        TImage;                                         {136}
    btn_cross_down_right:       TImage;                                         {137}

    btn_ctrlr_overlay_up:       TImage;                                         {18}
    btn_ctrlr_overlay_left:     TImage;                                         {19}
    btn_ctrlr_overlay_right:    TImage;                                         {20}
    btn_ctrlr_overlay_down:     TImage;                                         {21}

{ * LCD SCREEN GRAPHICS (SPRITES)                                            * }

    // on-screen panel (background, alarm, time/score, game a/b, etc.)
    im_panel_alarm_1: TImage; im_panel_alarm_2: TImage; im_panel_alarm_3: TImage; {22-24}

    im_panel_game_a: TImage; im_panel_game_b: TImage;                           {25-26}

    im_panel_time_am: TImage; im_panel_time_pm: TImage; im_panel_time_colon: TImage;{27-29}

    im_panel_life_1: TImage; im_panel_life_2: TImage; im_panel_life_3: TImage;  {30-32}

    im_panel_digit_pos_1: TImage; im_panel_digit_pos_2: TImage; im_panel_digit_pos_3: TImage; im_panel_digit_pos_4: TImage; {33-36}
    im_panel_number_0: TImage; im_panel_number_1: TImage; im_panel_number_2: TImage; im_panel_number_3: TImage; im_panel_number_4: TImage;
    im_panel_number_5: TImage; im_panel_number_6: TImage; im_panel_number_7: TImage; im_panel_number_8: TImage; im_panel_number_9: TImage; {37-46}

    // on-screen items
    im_heart_1: TImage; im_heart_2: TImage;                                     {47-48}

    im_girder_1: TImage; im_girder_2: TImage;                                   {49-50}

    im_wire_1: TImage; im_wire_2: TImage; im_wire_3: TImage; im_wire_4: TImage; {51-54}

    im_crane_1: TImage; im_crane_2: TImage; im_crane_3: TImage;                 {55-57}

    im_hook_1: TImage; im_hook_2: TImage; im_hook_3: TImage; im_hook_4: TImage; im_hook_5: TImage; {58-62}

    im_lever_1: TImage; im_lever_2: TImage;                                     {63-64}

    {                     }  im_kong_2a: TImage;      {                     }  im_kong_4a: TImage;      {                     }  im_kong_6a: TImage;      im_kong_7: TImage;       {65-68}
    {                     }  im_kong_2b: TImage;      {                     }  im_kong_4b: TImage;      {                     }  im_kong_6b: TImage;      {                     }  {69-71}
    {                     }  im_kong_2c: TImage;      {                     }  im_kong_4c: TImage;      {                     }  im_kong_6c: TImage;      {                     }  {72-74}

    im_iron_01_04: TImage;   im_iron_02_04: TImage;   im_iron_03_04: TImage;   im_iron_04_04: TImage;   im_iron_05_04: TImage;   {                     }  {                     }  {75-79}

    {                     }  im_barrel_02_08: TImage; {                     }  im_barrel_04_08: TImage; {                     }  im_barrel_06_08: TImage; {                     }  {80-82}
    {                     }  im_barrel_02_07: TImage; {                     }  im_barrel_04_07: TImage; {                     }  im_barrel_06_07: TImage; {                     }  {83-85}
    {                     }  im_barrel_02_06: TImage; {                     }  im_barrel_04_06: TImage; {                     }  im_barrel_06_06: TImage; {                     }  {86-88}
    im_barrel_01_05: TImage; im_barrel_02_05: TImage; im_barrel_03_05: TImage; im_barrel_04_05: TImage; im_barrel_05_05: TImage; im_barrel_06_05: TImage; {                     }  {89-94}
    im_barrel_01_04: TImage; {                     }  {                     }  {                     }  {                     }  {                     }  {                     }  {95}
    im_barrel_01_03: TImage; im_barrel_02_03: TImage; im_barrel_03_03: TImage; im_barrel_04_03: TImage; im_barrel_05_03: TImage; im_barrel_06_03: TImage; {                     }  {96-101}
    im_barrel_01_01: TImage; im_barrel_02_01: TImage; im_barrel_03_01: TImage; im_barrel_04_01: TImage; im_barrel_05_01: TImage; im_barrel_06_01: TImage; {                     }  {102-107}

    im_mario_01_07a: TImage; {                     }  im_mario_03_07: TImage;  {                     }  im_mario_05_07: TImage;  im_mario_06_07a: TImage; {                     }  {108-111}
    im_mario_01_07b: TImage; {                     }  {                     }  {                     }  {                     }  im_mario_06_07b: TImage; {                     }  {112-113}
    im_mario_01_07c: TImage; {                     }  {                     }  {                     }  {                     }  im_mario_06_07c: TImage; {                     }  {114-115}
    {                     }  {                     }  {                     }  {                     }  {                     }  im_mario_06_07d: TImage; {                     }  {116}
    {                     }  {                     }  {                     }  {                     }  {                     }  im_mario_06_07e: TImage; {                     }  {117}
    im_mario_01_05: TImage;  {                     }  {                     }  {                     }  {                     }  {                     }  {                     }  {118}
    im_mario_01_04: TImage;  {                     }  im_mario_03_04: TImage;  im_mario_04_04: TImage;  {                     }  {                     }  {                     }  {119-121}
    im_mario_01_03: TImage;  im_mario_02_03: TImage;  im_mario_03_03: TImage;  im_mario_04_03: TImage;  im_mario_05_03: TImage;  {                     }  {                     }  {122-126}
    im_mario_01_02: TImage;  {                     }  {                     }  im_mario_04_02: TImage;  {                     }  {                     }  {                     }  {127-128}
    im_mario_01_01: TImage;  im_mario_02_01: TImage;  im_mario_03_01: TImage;  im_mario_04_01: TImage;  im_mario_05_01: TImage;                                                    {129-133}

{ * TIMERS                                                                   * }
    Timer_Bonus:          TTimer;
    Timer_Bonus_Chance:   TTimer;
    Timer_Bonus_Complete: TTimer;
    Timer_Clock:          TTimer;
    Timer_Game:           TTimer;
    Timer_Game_Over:      TTimer;
    Timer_Hook:           TTimer;
    Timer_Mario_Move:     TTimer;
    Timer_Miss:           TTimer;
    Timer_Miss_Jump:      TTimer;
    Timer_Score_Blink:    TTimer;

{ * POPUP MENU                                                               * }
    pm_0_menu:          TPopupMenu;
    pm_1_about:         TMenuItem;
    pm_2_readme:        TMenuItem;
    pm_3_whatsnew:      TMenuItem;
    pm_4_manual:        TMenuItem;
    pm_5_n1:            TMenuItem;
    pm_6_config:        TMenuItem;
    pm_7_keys:          TMenuItem;
    pm_8_hiscore:       TMenuItem;
    pm_9_n2:            TMenuItem;
  {}pm_10_fullscreen:   TMenuItem;
  {}pm_11_splashscreen: TMenuItem;
  {}pm_12_controller:   TMenuItem;
  {}pm_13_n3:           TMenuItem;
  {}pm_14_exit:         TMenuItem;

{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
    procedure FormCreate                    (Sender: TObject);
    procedure FormActivate                  (Sender: TObject);
    procedure FormClose                     (Sender: TObject; var Action: TCloseAction);

{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
    procedure Vid_Display_Ctrl_Overlay;

    procedure Vid_Show_All                  (b: boolean);

    procedure Vid_Show_All_Mario            (b: boolean);

    procedure Vid_Show_Panel_Lives          (var p: integer);
    procedure Vid_Show_Panel_Score          (var p: integer; b: boolean);

    procedure Vid_Show_Hook                 (p: integer);
    procedure Vid_Show_Kong                 (var p, q: integer);
    procedure Vid_Show_Mario                (var p, q: integer; b: boolean);
    procedure Vid_Show_Mario0107Hand        (var p: integer);
    procedure Vid_Show_Wires                (var p: integer);

{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
    procedure Snd_Play                      (pfs: PFSoundSample);
    procedure Snd_Silence;

{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
    procedure im_background_closedClick     (Sender: TObject);

    procedure im_background_openMouseDown   (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure im_background_openMouseMove   (Sender: TObject;                       Shift: TShiftState; X, Y: Integer);

    procedure btn_game_a_topMouseDown       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_a_topMouseUp         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseDown       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseUp         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_acl_topMouseDown          (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_acl_topMouseUp            (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_jump_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_jump_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_ctrlr_overlay_upMouseDown (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_upMouseUp   (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_rightMouseDown(Sender:TObject;Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_rightMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_leftMouseDown(Sender:TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_leftMouseUp (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_downMouseDown(Sender:TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_downMouseUp (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure FormKeyDown                   (Sender: TObject; var Key: Word;        Shift: TShiftState);
    procedure FormKeyUp                     (Sender: TObject; var Key: Word;        Shift: TShiftState);

{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
    procedure Inp_game_a_down;              procedure Inp_game_a_up;
    procedure Inp_game_b_down;              procedure Inp_game_b_up;
    procedure Inp_time_down;                procedure Inp_time_up;
    procedure Inp_acl_down;                 procedure Inp_acl_up;

    procedure Inp_right_down;               procedure Inp_right_up;
    procedure Inp_down_down;                procedure Inp_down_up;
    procedure Inp_left_down;                procedure Inp_left_up;
    procedure Inp_up_down;                  procedure Inp_up_up;

    procedure Inp_jump_down;                procedure Inp_jump_up;

{ * PROCEDURES: GAME ROUTINES                                                * }
    procedure Gam_Stop_All_Timers;

    procedure Gam_Mario_Jumps_In;
    procedure Gam_Set_Game_Start;
    procedure Gam_Set_Mode;
    procedure Gam_Set_TimerGame_Interval;

{ * PROCEDURES: TIMERS                                                       * }
    procedure Timer_BonusTimer              (Sender: TObject);
    procedure Timer_Bonus_ChanceTimer       (Sender: TObject);
    procedure Timer_Bonus_CompleteTimer     (Sender: TObject);
    procedure Timer_ClockTimer              (Sender: TObject);
    procedure Timer_GameTimer               (Sender: TObject);
    procedure Timer_Game_OverTimer          (Sender: TObject);
    procedure Timer_HookTimer               (Sender: TObject);
    procedure Timer_Mario_MoveTimer         (Sender: TObject);
    procedure Timer_MissTimer               (Sender: TObject);
    procedure Timer_Miss_JumpTimer          (Sender: TObject);
    procedure Timer_Score_BlinkTimer        (Sender: TObject);

{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
    procedure pm_0_menuPopup                (Sender: TObject);
    procedure pm_1_aboutClick               (Sender: TObject);
    procedure pm_2_readmeClick              (Sender: TObject);
    procedure pm_3_whatsnewClick            (Sender: TObject);
    procedure pm_4_manualClick              (Sender: TObject);
    procedure pm_6_configClick              (Sender: TObject);
    procedure pm_7_keysClick                (Sender: TObject);
    procedure pm_8_hiscoreClick             (Sender: TObject);
  {}procedure pm_10_fullscreenClick         (Sender: TObject);
  {}procedure pm_11_splashscreenClick       (Sender: TObject);
  {}procedure pm_12_controllerClick         (Sender: TObject);
  {}procedure pm_14_exitClick               (Sender: TObject);

{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
    procedure btn_game_menu_topClick        (Sender: TObject);

    procedure la_menu_itemMouseEnter        (Sender: TObject);
    procedure la_menu_itemMouseLeave        (Sender: TObject);
    procedure la_menu_itemMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure Vid_Display_Game_Menu         (b: boolean);
  end;

const
{ introduced for use with HH2 / now incorporated into main source }

  HH2_CONTENT_NAME    = 'Donkey Kong (Nintendo, Game & Watch Multi Screen)';
  HH2_CONTENT_AUTHOR  = 'Luca "MADrigal" Antignano';
  HH2_CONTENT_EMAIL   = 'lucantignano@gmail.com';
  HH2_CONTENT_VERSION = '5.01';
  HH2_CONTENT_DATE    = '2022/03/14';
  HH2_CONTENT_WEBSITE = 'www.madrigaldesign.it/sim';

  { constants for all builds }
  
  APP_NAME    = HH2_CONTENT_NAME + ' Simulator';                                // application's name (appears on taskbar and Splash screen)
{}APP_VERSION = 'Version ' + HH2_CONTENT_VERSION + ' * Graphics improvements *';// current version (appears on Splash screen)
{}APP_DATE    = 'Released by ' + HH2_CONTENT_AUTHOR + ' on 14th March, 2022';   // release date (appears on Splash screen)

  URL_EMAIL   = 'mailto:' + HH2_CONTENT_EMAIL;                                  // link to the email address
  URL_PAYPAL  = 'https://www.paypal.me/LucaAntignano';                          // link to the Paypal donation page
  URL_WEBSITE = 'http://' + HH2_CONTENT_WEBSITE + '/';                          // link to the website

  REG_KEY     =  'Software\MADrigal\DKong50';                                   // registry key

  HINT_CLICK  = 'click or press';
  HINT_OPEN   = 'to open the game';
  HINT_GAME_A = 'to start game A (hold to view highscore)';
  HINT_GAME_B = 'to start game B (hold to view highscore)';
  HINT_TIME   = 'to view current time (hold to view alarm time)';
  HINT_ACL    = 'to reset the game';
  HINT_ALARM  = 'button disabled';
  HINT_CROSS  = 'activate the controller overlays or use';

  TIMER_MARIO_DELAY_SHORT = 90;
  TIMER_MARIO_DELAY_LONG  = 330;

{}MENU_ITEMS = 14;

var
{ * main objects * }
  Form1:                   TForm1;                                              // main form
  bStartup:                boolean;                                             // manages splash screen at startup
  iMouseX, iMouseY:        integer;                                             // cursor coordinates (to drag main form)
  iReg:                    TRegIniFile;                                         // system registry (to save high-scores)

{ * sound management * }
  pfs_barrel,
  pfs_bonus,
  pfs_fall,
  pfs_jump,
  pfs_kongfall,
  pfs_gameover,
  pfs_miss,
  pfs_step:                PFSoundSample;                                       // sound samples

  bSound:                  boolean;                                             // sound enabled? (depending on selected driver on splash screen)

{ * arrays of images * }
  aIm_panel_digit_pos:     array[1..4]       of TImage;                         // empty sprites to host score/time digits
  aIm_panel_alarm:         array[1..3]       of TImage;                         // sprites for alarm (unused)
  aIm_panel_lives:         array[1..3]       of TImage;                         // sprites for lives panel
  aIm_panel_number:        array[0..9]       of TImage;                         // digits for score/time

  aIm_girder,
  aIm_heart,
  aIm_lever:               array[1..2]       of TImage;
  aIm_crane:               array[1..3]       of TImage;
  aIm_wire:                array[1..4]       of TImage;
  aIm_hook:                array[1..5]       of TImage;

  aIm_iron:                array[1..5]       of TImage;
  aIm_kong:                array[2..6, 1..3] of TImage;
  aIm_mario:               array[1..5, 1..7] of TImage;
  aIm_mario_01_07:         array[1..2]       of TImage;
  aIm_mario_06_07:         array[1..5]       of TImage;
  aIm_barrel:              array[1..6, 1..8] of TImage;

{ * arrays of mixed types *}
  aBarrel:                 array[1..6, 1..8] of integer;

  aKeys:                   array[1..9]       of record                          // keys management (values)
                                                  int:  integer;
                                                  word: word;
                                                end;

  aKeyHints:               array[1..44] of string;                              // keys management (hint on buttons)

  aMenuItem:               array[1..MENU_ITEMS] of TMenuItem;                   // popup menu items
  aMenuLabels:             array[1..MENU_ITEMS] of TLabel;                      // on-screen game menu labels

{ * integers and booleans * }
  iMode,                                                                        // 0=closed, 1/2=ACL 1/2, 3=alarm, 4=time, 5/7=hiscore A/B, 6/8=game A/B, 9=game over
  iScore,                                                                       // score=0-999
  iHiScoreA,                                                                    // hiscore for Game A=0-999
  iHiScoreB,                                                                    // hiscore for Game B=0-999
  iScoreMultiplier,                                                             // given points=1 (normal) or 2 (chance-time)
  iScoreChanceNext,                                                             // 300, 1300, 2300, etc.

  iLives,                                                                       // 0-3
  iWires,                                                                       // 0-4
  iHookPosX,                                                                    // 1-5

  iMarioPosX,                                                                   // 1-5
  iMarioPosY,                                                                   // 1-7
  iMario0107Hand,                                                               // 0=hand off, 1=hand down, 2=hand up
  iKongPosX,                                                                    // 2, 4 or 6
  iKongPosY,                                                                    // 2=arms up, 3=arms down

  iMaxRandomBarrel,                                                             // depending on Mario's y coordinate
  iMaxRandomIron,                                                               // depending on Mario's y coordinate

  iCounterGame,                                                                 // used in TimerGame (counts how many ticks occured)
  iCounterBonus,                                                                // used in TimerBonusComplete (counts how many points to add to score)
  iCounterBonusComplete,                                                        // used in TimerBonus (counts how many points to add to score)

  iBonusChanceReprise                                                           // which timer was stopped at 300 points score? 1=TimerGame, 2=TimerBonus, 3=TimerBonusComplete
                         : integer;

{}bFullscreenMode,                                                              // true if video mode is fullscreen (false if "shaped game")
  bGameMenuOpen,                                                                // true if the on-screen menu is visible
  bDisplayCtrlOverlays,                                                         // true if the controller overlay should be displayed

  bController,                                                                  // true while player can actually operate the main character
  bArrowKeyPressed,                                                             // true while player keeps any arrow key pressed
  bJumpKeyPressed                                                               // true while player keeps jump key pressed
                         : boolean;

  // Game variables: loops
  iLoopGame,                                                                    // used in TimerGame (1 to 4 then 1 again)
  iLoopMarioMove,                                                               // used in TimerMarioMove, Mario goes 1=left, 2=up, 3=right, 4=down
  iLoopHook,                                                                    // used in TimerHook
  iLoopScoreBlink,                                                              // used in TimerScoreBlink
  iLoopMiss,                                                                    // used in TimerMiss, TimerMissJump and TimerMissIron
  iLoopGameOver,                                                                // used in TimerGameOver
  iLoopBonus,                                                                   // used in TimerBonus
  iLoopBonusComplete,                                                           // used in TimerBonusComplete
  iLoopBonusChance                                                              // used in TimerBonusChance
                         : integer;

implementation

uses
  about, gfxinit, keys, manual, splash;

{$r *.dfm}










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.FormCreate(Sender: TObject);
{ sprites on form are associated to arrays of images, needed to "index" them   }
{ variables are initialized: system registry, randomize, game mode, etc.       }
  var
    i: integer;
begin
  // initializes application name (for Windows taskbar)
  Application.Title := APP_NAME;

{}// sets game off the screen while graphics initialise
{}Top  := - Screen.Height;
{}Left := - Screen.Width;

  // on-screen panel
  aIm_panel_alarm[1] := im_panel_alarm_1; aIm_panel_alarm[2] := im_panel_alarm_2; aIm_panel_alarm[3] := im_panel_alarm_3;
  aIm_panel_lives[1] := im_panel_life_1; aIm_panel_lives[2] := im_panel_life_2; aIm_panel_lives[3] := im_panel_life_3;
  aIm_panel_digit_pos[1] := im_panel_digit_pos_1; aIm_panel_digit_pos[2] := im_panel_digit_pos_2; aIm_panel_digit_pos[3] := im_panel_digit_pos_3; aIm_panel_digit_pos[4] := im_panel_digit_pos_4;

  // numbers
  aIm_panel_number[0] := im_panel_number_0; aIm_panel_number[1] := im_panel_number_1; aIm_panel_number[2] := im_panel_number_2; aIm_panel_number[3] := im_panel_number_3; aIm_panel_number[4] := im_panel_number_4;
  aIm_panel_number[5] := im_panel_number_5; aIm_panel_number[6] := im_panel_number_6; aIm_panel_number[7] := im_panel_number_7; aIm_panel_number[8] := im_panel_number_8; aIm_panel_number[9] := im_panel_number_9;

  // on-screen items
  aIm_heart[1] := im_heart_1; aIm_heart[2] := im_heart_2;

  aIm_girder[1] := im_girder_1; aIm_girder[2] := im_girder_2;

  aIm_wire[1] := im_wire_1; aIm_wire[2] := im_wire_2; aIm_wire[3] := im_wire_3; aIm_wire[4] := im_wire_4;

  aIm_crane[1] := im_crane_1; aIm_crane[2] := im_crane_2; aIm_crane[3] := im_crane_3;

  aIm_hook[1]  := im_hook_1; aIm_hook[2] := im_hook_2; aIm_hook[3] := im_hook_3; aIm_hook[4] := im_hook_4; aIm_hook[5] := im_hook_5;

  aIm_lever[1] := im_lever_1; aIm_lever[2] := im_lever_2;

  {                                 }  aIm_kong[2, 1] := im_kong_2a;       {                                 }   aIm_kong[4, 1] := im_kong_4a;        {                                 }  aIm_kong[6, 1] := im_kong_6a;
  {                                 }  aIm_kong[2, 2] := im_kong_2b;       {                                 }   aIm_kong[4, 2] := im_kong_4b;        {                                 }  aIm_kong[6, 2] := im_kong_6b;
  {                                 }  aIm_kong[2, 3] := im_kong_2c;       {                                 }   aIm_kong[4, 3] := im_kong_4c;        {                                 }  aIm_kong[6, 3] := im_kong_6c;

  aIm_iron[1] := im_iron_01_04;        aIm_iron[2] := im_iron_02_04;        aIm_iron[3] := im_iron_03_04;        aIm_iron[4] := im_iron_04_04;        aIm_iron[5] := im_iron_05_04;        {                                 }

  {                                 }  aIm_barrel[2, 8] := im_barrel_02_08; {                                 }  aIm_barrel[4, 8] := im_barrel_04_08; {                                 }  aIm_barrel[6, 8] := im_barrel_06_08;
  {                                 }  aIm_barrel[2, 7] := im_barrel_02_07; {                                 }  aIm_barrel[4, 7] := im_barrel_04_07; {                                 }  aIm_barrel[6, 7] := im_barrel_06_07;
  {                                 }  aIm_barrel[2, 6] := im_barrel_02_06; {                                 }  aIm_barrel[4, 6] := im_barrel_04_06; {                                 }  aIm_barrel[6, 6] := im_barrel_06_06;
  aIm_barrel[1, 5] := im_barrel_01_05; aIm_barrel[2, 5] := im_barrel_02_05; aIm_barrel[3, 5] := im_barrel_03_05; aIm_barrel[4, 5] := im_barrel_04_05; aIm_barrel[5, 5] := im_barrel_05_05; aIm_barrel[6, 5] := im_barrel_06_05;
  aIm_barrel[1, 4] := im_barrel_01_04; {                                 }  {                                 }  {                                 }  {                                 }  {                                 }
  aIm_barrel[1, 3] := im_barrel_01_03; aIm_barrel[2, 3] := im_barrel_02_03; aIm_barrel[3, 3] := im_barrel_03_03; aIm_barrel[4, 3] := im_barrel_04_03; aIm_barrel[5, 3] := im_barrel_05_03; aIm_barrel[6, 3] := im_barrel_06_03;
  aIm_barrel[1, 1] := im_barrel_01_01; aIm_barrel[2, 1] := im_barrel_02_01; aIm_barrel[3, 1] := im_barrel_03_01; aIm_barrel[4, 1] := im_barrel_04_01; aIm_barrel[5, 1] := im_barrel_05_01; aIm_barrel[6, 1] := im_barrel_06_01;

  aIm_mario[1, 7] := im_mario_01_07a;  {                                 }  {                                 }  {                                 }  {                                 }
  aIm_mario[1, 5] := im_mario_01_05;   {                                 }  aIm_mario[3, 7] := im_mario_03_07;   {                                 }  aIm_mario[5, 7] := im_mario_05_07;
  aIm_mario[1, 4] := im_mario_01_04;   {                                 }  aIm_mario[3, 4] := im_mario_03_04;   aIm_mario[4, 4] := im_mario_04_04;   {                                 }
  aIm_mario[1, 3] := im_mario_01_03;   aIm_mario[2, 3] := im_mario_02_03;   aIm_mario[3, 3] := im_mario_03_03;   aIm_mario[4, 3] := im_mario_04_03;   aIm_mario[5, 3] := im_mario_05_03;
  aIm_mario[1, 2] := im_mario_01_02;   {                                 }  {                                 }  aIm_mario[4, 2] := im_mario_04_02;   {                                 }
  aIm_mario[1, 1] := im_mario_01_01;   aIm_mario[2, 1] := im_mario_02_01;   aIm_mario[3, 1] := im_mario_03_01;   aIm_mario[4, 1] := im_mario_04_01;   aIm_mario[5, 1] := im_mario_05_01;

  aIm_mario_01_07[1] := im_mario_01_07b; aIm_mario_01_07[2] := im_mario_01_07c;

  aIm_mario_06_07[1] := im_mario_06_07a; aIm_mario_06_07[2] := im_mario_06_07b; aIm_mario_06_07[3] := im_mario_06_07c; aIm_mario_06_07[4] := im_mario_06_07d; aIm_mario_06_07[5] := im_mario_06_07e;

  // registry keys management
  iReg := TregIniFile.Create(REG_KEY);                                          // system registry init

{}// init graphics
{}bFullscreenMode := iReg.ReadBool('', 'FullscreenMode', true);
{}pm_10_fullscreen.Checked := bFullscreenMode;
{}Gfxinit.Gfx_Game_Init;
{}bGameMenuOpen := false;

{}// initialises the "hints" array
  for i := 1 to 10 do
    aKeyHints[i] := '''' + Chr(i + 47) + '''';

  for i := 11 to 36 do
    aKeyHints[i] := '''' + Chr(i + 54) + '''';

  aKeyHints[37] := '''up arrow'' key';
  aKeyHints[38] := '''down arrow'' key';
  aKeyHints[39] := '''left arrow'' key';
  aKeyHints[40] := '''right arrow'' key';
  aKeyHints[41] := '''alt'' key';
  aKeyHints[42] := '''ctrl'' key';
  aKeyHints[43] := '''shift'' key';
  aKeyHints[44] := '''spacebar''';

  // reads values from system registry into the inputs array
  aKeys[1].int := iReg.ReadInteger('', 'Key1', 2);                              // "Game A" (default: "1")
  aKeys[2].int := iReg.ReadInteger('', 'Key2', 3);                              // "Game B" (default: "2")
  aKeys[3].int := iReg.ReadInteger('', 'Key3', 4);                              // "Time"   (default: "3")
  aKeys[4].int := iReg.ReadInteger('', 'Key4', 6);                              // "ACL"    (default: "5")
  aKeys[5].int := iReg.ReadInteger('', 'Key5', 37);                             // "Up"     (default: "UP")
  aKeys[6].int := iReg.ReadInteger('', 'Key6', 39);                             // "Left"   (default: "LEFT")
  aKeys[7].int := iReg.ReadInteger('', 'Key7', 40);                             // "Right"  (default: "RIGHT")
  aKeys[8].int := iReg.ReadInteger('', 'Key8', 38);                             // "Down"   (default: "DOWN")
  aKeys[9].int := iReg.ReadInteger('', 'Key9', 42);                             // "Jump"   (default: "CTRL")

  // writes values into the system registry (in case no pre-existing values
  // stored), then assigns key values to the key-inputs array
  for i := 1 to 9 do
    begin
      iReg.WriteInteger('', 'Key' + inttostr(i), aKeys[i].int);
      KeysForm.AssignWord(i, aKeys[i].int);
    end;

  // sets button images hints
  im_background_closed.Hint    := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int] + ' ' + HINT_OPEN;   // "Open"

  btn_game_a_top.Hint          := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_GAME_A; // "Game A"
  btn_game_b_top.Hint          := HINT_CLICK + ' ' + aKeyHints[aKeys[2].int] + ' ' + HINT_GAME_B; // "Game B"
  btn_time_top.Hint            := HINT_CLICK + ' ' + aKeyHints[aKeys[3].int] + ' ' + HINT_TIME;   // "Time"
  btn_acl_top.Hint             := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int] + ' ' + HINT_ACL;    // "ACL"
  btn_alarm_top.Hint           := HINT_ALARM;                                                     // "Alarm"

  btn_cross_top.Hint           := HINT_CROSS + ' ' + aKeyHints[aKeys[5].int] + ', '               // "Up"
                                                   + aKeyHints[aKeys[6].int] + ', '               // "Left"
                                                   + aKeyHints[aKeys[7].int] + ', '               // "Right"
                                                   + aKeyHints[aKeys[8].int];                     // "Down"

  btn_jump_top.Hint            := HINT_CLICK + ' ' + aKeyHints[aKeys[9].int];                     // "Jump"

  btn_ctrlr_overlay_up.Hint    := HINT_CLICK + ' ' + aKeyHints[aKeys[5].int];                     // "Up"
  btn_ctrlr_overlay_left.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[6].int];                     // "Left"
  btn_ctrlr_overlay_right.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[7].int];                     // "Right"
  btn_ctrlr_overlay_down.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[8].int];                     // "Down"

  // popup menu array association
  aMenuItem[1]  := pm_1_about;
  aMenuItem[2]  := pm_2_readme;
  aMenuItem[3]  := pm_3_whatsnew;
  aMenuItem[4]  := pm_4_manual;
  aMenuItem[5]  := pm_5_n1;
  aMenuItem[6]  := pm_6_config;
  aMenuItem[7]  := pm_7_keys;
  aMenuItem[8]  := pm_8_hiscore;
  aMenuItem[9]  := pm_9_n2;
{}aMenuItem[10] := pm_10_fullscreen;
{}aMenuItem[11] := pm_11_splashscreen;
{}aMenuItem[12] := pm_12_controller;
{}aMenuItem[13] := pm_13_n3;
{}aMenuItem[14] := pm_14_exit;

  // controller overlay setting
{}pm_12_controller.Checked  := iReg.ReadBool('', 'CtrlrOverlay', true);
{}bDisplayCtrlOverlays := pm_12_controller.Checked;

  // variables init
  DoubleBuffered := true;                                                       // enables double buffering rendering

  iHiScoreA := iReg.ReadInteger('', 'HiScore1', iHiScoreA);                     // reads value from system registry into iHiScoreA
  iHiScoreB := iReg.ReadInteger('', 'HiScore2', iHiScoreB);                     // reads value from system registry into iHiScoreB

  bController := false;                                                         // main character can't move
  bArrowKeyPressed := false;
  bJumpKeyPressed  := false;

  Randomize;                                                                    // initializes the random numbers generator
end;





procedure TForm1.FormActivate(Sender: TObject);
{ sends focus to the splash screen at startup                                  }
begin
  if bStartup then
    begin
      bStartup := false;
      SplashForm.Show;
    end;
end;





procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
{ frees memory before shutting down                                            }
begin
  FSOUND_Sample_Free(pfs_barrel);   FSOUND_Sample_Free(pfs_bonus);
  FSOUND_Sample_Free(pfs_fall);     FSOUND_Sample_Free(pfs_jump);
  FSOUND_Sample_Free(pfs_kongfall); FSOUND_Sample_Free(pfs_gameover);
  FSOUND_Sample_Free(pfs_miss);     FSOUND_Sample_Free(pfs_step);

  FSOUND_Close;

{}iReg.Destroy;
  Action := caFree;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Vid_Display_Ctrl_Overlay;
{ displays or hides the controller overlay images                              }
begin
  btn_ctrlr_overlay_up.Visible    := bDisplayCtrlOverlays and im_background_open.Visible;
  btn_ctrlr_overlay_left.Visible  := bDisplayCtrlOverlays and im_background_open.Visible;
  btn_ctrlr_overlay_right.Visible := bDisplayCtrlOverlays and im_background_open.Visible;
  btn_ctrlr_overlay_down.Visible  := bDisplayCtrlOverlays and im_background_open.Visible;

  btn_cross_top.Visible := (not bDisplayCtrlOverlays) and im_background_open.Visible;
end;





procedure TForm1.Vid_Show_All(b: boolean);
{ displays/hides all sprites                                                   }
  var
    i, j: integer;
begin
  for i := 1 to 4 do
    aIm_panel_digit_pos[i].Visible := true;

  if b then
    begin
      aIm_panel_digit_pos[1].Picture := aIm_panel_number[1].Picture;
      aIm_panel_digit_pos[2].Picture := aIm_panel_number[2].Picture;
      aIm_panel_digit_pos[3].Picture := aIm_panel_number[0].Picture;
      aIm_panel_digit_pos[4].Picture := aIm_panel_number[0].Picture;
    end
  else
    for i := 1 to 4 do
      aIm_panel_digit_pos[1].Picture := nil;

  im_panel_game_a.Visible := b;
  im_panel_game_b.Visible := b;
  im_panel_time_am.Visible := b;
  im_panel_time_pm.Visible := false;
  im_panel_time_colon.Visible := b;

  for i := 1 to 3 do
    aIm_panel_alarm[i].Visible := b;

  i := Ord(b) * 3; Vid_Show_Panel_Lives(i);

  // in-game sprites
  for i := 1 to 2 do aIm_heart[i].Visible := b;
  for i := 1 to 2 do aIm_girder[i].Visible := b;
  for i := 1 to 4 do aIm_wire[i].Visible := b;
  for i := 1 to 3 do aIm_crane[i].Visible := b;
  for i := 1 to 5 do aIm_hook[i].Visible := b;
  for i := 1 to 2 do aIm_lever[i].Visible := b;
  for i := 1 to 5 do aIm_iron[i].Visible := b;

  for i := 1 to 3 do
    for j := 1 to 3 do aIm_kong[i * 2, j].Visible := b;

  im_kong_7.Visible := b;

  for i := 1 to 3 do
    for j := 6 to 8 do
      aIm_barrel[i * 2, j].Visible := b;

  for i := 1 to 6 do
    for j := 1 to 3 do
      aIm_barrel[i, j * 2 - 1].Visible := b;

  aIm_barrel[1, 4].Visible := b;

  Vid_Show_All_Mario(b);
end;





procedure TForm1.Vid_Show_All_Mario(b: boolean);
{ displays/hides all Mario sprites                                             }
  var
    i, j: integer;
begin
  for i := 1 to 3 do aIm_mario[i * 2 - 1, 7].Visible    := b;
  for j := 4 to 5 do aIm_mario[1, j].Visible            := b;
  for i := 3 to 4 do aIm_mario[i, 4].Visible            := b;
  for i := 1 to 5 do
    for j := 1 to 2 do aIm_mario[i, j * 2 - 1].Visible  := b;
  aIm_mario[1, 2].Visible                               := b;
  aIm_mario[4, 2].Visible                               := b;
  for i := 1 to 2 do aIm_mario_01_07[i].Visible         := b;
  for i := 1 to 5 do aIm_mario_06_07[i].Visible         := b;
end;





procedure TForm1.Vid_Show_Hook(p: integer);
{ displays/hides hook (0 or 1 to 5)                                            }
  var i: integer;
begin
  for i := 1 to 5 do
    aIm_hook[i].Visible := i = p; // works if p = 0, too
end;





procedure TForm1.Vid_Show_Kong(var p, q: integer);
{ displays/hides Donkey Kong in 'p' column (2, 4 or 6) with arms up or down    }
{ body + arms: visible if p<>0, hidden if p=0                                  }
{ arms: down if q=2, up if q=2                                                 }
  var
    i, j: integer;
begin
  if (p = 0) then
    for i := 1 to 3 do
      for j := 1 to 3 do
        aIm_kong[i * 2, j].Visible := false
  else
    begin
      for i := 1 to 3 do
        begin
          aIm_kong[i * 2, 1].Visible := (i * 2 = p);
          for j := 2 to 3 do
            aIm_kong[i * 2, j].Visible := (i * 2 = p) and (j = q);
          end;
    end;
end;





procedure TForm1.Vid_Show_Mario(var p, q: integer; b: boolean);
{ displays/hides Mario in 'p, q' position                                      }
begin
  aIm_mario[p, q].Visible := b;
end;





procedure TForm1.Vid_Show_Mario0107Hand(var p: integer);
{ displays/hides Mario's hand when he's in (1,7)                               }
{ values: 0 (hidden), 1 (hand down), 2( hand up)                               }
begin
  case p of
    0: begin aIm_mario_01_07[1].Visible := false; aIm_mario_01_07[2].Visible := false; end;
    1: begin aIm_mario_01_07[1].Visible := true;  aIm_mario_01_07[2].Visible := false; end;
    2: begin aIm_mario_01_07[1].Visible := false; aIm_mario_01_07[2].Visible := true;  end;
  end; // case..of
end;





procedure TForm1.Vid_Show_Wires(var p: integer);
{ displays/hides wires (0 or 1 to 4)                                           }
  var i: integer;
begin
  for i := 1 to 4 do
    aIm_wire[i].Visible := p >= i;
end;





procedure TForm1.Vid_Show_Panel_Lives(var p: integer);
{ displays/hides lives left (1 to 3)                                           }
  var i: integer;
begin
  for i := 1 to 3 do
    aIm_panel_lives[i].Visible := p >= i;
end;





procedure TForm1.Vid_Show_Panel_Score(var p: integer; b: boolean);
{ transforms score/high-score into sprites, and displays/hides them            }
  var
    i, t: integer;
begin
  t := p;

  while t >= 1000 do                                                            // displays "1000" as "...0"
    Dec(t, 1000);

  aIm_panel_digit_pos[1].Picture := nil;                                        // thousands (always hidden)

  if b then                                                                     // displays or hides score/high-score?
    begin                                                                         // displays
      if t > 99 then aIm_panel_digit_pos[2].Picture := aIm_panel_number[t div 100 mod 10].picture else aIm_panel_digit_pos[2].Picture := nil; // hundreds
      if t >  9 then aIm_panel_digit_pos[3].Picture := aIm_panel_number[t div  10 mod 10].picture else aIm_panel_digit_pos[3].Picture := nil; // tens
                     aIm_panel_digit_pos[4].Picture := aIm_panel_number[t         mod 10].picture;                                            // units
    end
  else                                                                            // hides
    for i := 2 to 4 do
      aIm_panel_digit_pos[i].picture := nil;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Snd_Play(pfs: PFSoundSample);
{ plays a certain sound sample                                                 }
begin
  if bSound then
    FSOUND_PlaySound(FSOUND_FREE, pfs);
end;





procedure TForm1.Snd_Silence;
{ stops any currently playing sound sample                                     }
begin
  if bSound then
    FSOUND_StopSound(FSOUND_ALL);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.im_background_closedClick(Sender: TObject);
{ opening the handheld game                                                    }
begin
  // background
  im_background_closed.Visible := false;
  im_background_open.Visible   := true;

  im_background_screen_1_top.Visible := true;
  im_background_screen_2_top.Visible := true;

  // buttons
  btn_game_a_top.Visible := true;
  btn_game_b_top.Visible := true;
  btn_time_top.Visible   := true;
  btn_acl_top.Visible    := true;
  btn_alarm_top.Visible  := true;

  btn_cross_top.Visible  := true;
  btn_jump_top.Visible   := true;

  // controller overlay
  Vid_Display_Ctrl_Overlay;

  // sets mode to "ACL/2"
  iMode := 2; Gam_Set_Mode;
end;





procedure TForm1.im_background_openMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: PRESS                                                }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (Button = mbLeft) then
    begin
      iMouseX := X;
      iMouseY := Y;
    end;
end;





procedure TForm1.im_background_openMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: MOVE                                                 }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (ssLeft in Shift) then
    begin
      Top  := Top  + Y - iMouseY;
      Left := Left + X - iMouseX;
    end;
end;





procedure TForm1.btn_game_a_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game A button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_a_down;
end;





procedure TForm1.btn_game_a_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game A button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_a_up;
end;





procedure TForm1.btn_game_b_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game B button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_b_down;
end;





procedure TForm1.btn_game_b_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game B button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_b_up;
end;





procedure TForm1.btn_time_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Time button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_time_down;
end;





procedure TForm1.btn_time_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Time button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_time_up;
end;





procedure TForm1.btn_acl_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ACL button: PRESS                                                            }
begin
  if Button = mbLeft then
    Inp_acl_down;
end;





procedure TForm1.btn_acl_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ACL button: RELEASE                                                          }
begin
  if Button = mbLeft then
    Inp_acl_up;
end;





procedure TForm1.btn_jump_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ JUMP button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_jump_down;
end;





procedure TForm1.btn_jump_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ JUMP button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_jump_up;
end;





procedure TForm1.btn_ctrlr_overlay_upMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ UP button: PRESS                                                             }
begin
  if Button = mbLeft then
    Inp_up_down;
end;





procedure TForm1.btn_ctrlr_overlay_upMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ UP button: RELEASE                                                           }
begin
  if Button = mbLeft then
    Inp_up_up;
end;





procedure TForm1.btn_ctrlr_overlay_leftMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ LEFT button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_left_down;
end;





procedure TForm1.btn_ctrlr_overlay_leftMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ LEFT button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_left_up;
end;





procedure TForm1.btn_ctrlr_overlay_rightMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ RIGHT button: PRESS                                                          }
begin
  if Button = mbLeft then
    Inp_right_down;
end;





procedure TForm1.btn_ctrlr_overlay_rightMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ RIGHT button: RELEASE                                                        }
begin
  if Button = mbLeft then
    Inp_right_up;

end;





procedure TForm1.btn_ctrlr_overlay_downMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ DOWN button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_down_down;
end;





procedure TForm1.btn_ctrlr_overlay_downMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ DOWN button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_down_up;
end;





procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: PRESS                                                              }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_down else                              // Game A
  if Key = aKeys[2].word then Inp_game_b_down else                              // Game B
  if Key = aKeys[3].word then Inp_time_down   else                              // Time
  if Key = aKeys[4].word then Inp_acl_down    else                              // ACL

  if Key = aKeys[5].word then Inp_up_down     else                              // Up
  if Key = aKeys[6].word then Inp_left_down   else                              // Left
  if Key = aKeys[7].word then Inp_right_down  else                              // Right
  if Key = aKeys[8].word then Inp_down_down   else                              // Down

  if Key = aKeys[9].word then Inp_jump_down;                                    // Jump
end;





procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: RELEASE                                                            }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_up else                                // Game A
  if Key = aKeys[2].word then Inp_game_b_up else                                // Game B
  if Key = aKeys[3].word then Inp_time_up   else                                // Time
  if Key = aKeys[4].word then Inp_acl_up    else                                // ACL

  if Key = aKeys[5].word then Inp_up_up     else                                // Up
  if Key = aKeys[6].word then Inp_left_up   else                                // Left
  if Key = aKeys[7].word then Inp_right_up  else                                // Right
  if Key = aKeys[8].word then Inp_down_up   else                                // Down

  if Key = aKeys[9].word then Inp_jump_up;                                      // Jump
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Inp_game_a_down;
{ Game A button: PRESS                                                         }
begin
  btn_game_a_down.Visible := true and im_background_open.Visible;

  if (iMode = 2) or                                                             // ACL/2     or
     (iMode = 4) or                                                             // Time      or
     (iMode = 9) then                                                           // Game over ->
    begin
      iMode := 5; Gam_Set_Mode;                                                   //         -> HiScore A
    end;
end;





procedure TForm1.Inp_game_a_up;
{ Game A button: RELEASE                                                       }
begin
  btn_game_a_down.Visible := false;

  if iMode = 5 then                                                             // HiScore A ->
    begin
      iMode := 6; Gam_Set_Mode;                                                   //         -> Game A
    end;
end;





procedure TForm1.Inp_game_b_down;
{ Game B button: PRESS                                                         }
begin
  btn_game_b_down.Visible := true and im_background_open.Visible;

  if (iMode = 2) or                                                             // ACL/2     or
     (iMode = 4) or                                                             // Time      or
     (iMode = 9) then                                                           // Game over ->
    begin
      iMode := 7; Gam_Set_Mode;                                                   //         -> HiScore B
    end;
end;





procedure TForm1.Inp_game_b_up;
{ Game B button: RELEASE                                                       }
begin
  btn_game_b_down.Visible := false;

  if iMode = 7 then                                                             // HiScore B ->
    begin
      iMode := 8; Gam_Set_Mode;                                                   //         -> Game B
    end;
end;





procedure TForm1.Inp_time_down;
{ Time button: PRESS                                                           }
begin
  btn_time_down.Visible := true and im_background_open.Visible;

  if (iMode = 2) or                                                             // ACL-2     or
     (iMode = 4) or                                                             // Time      or
     (iMode = 9) then                                                           // Game over ->
    begin
      iMode := 3; Gam_Set_Mode;                                                   //         -> Alarm
    end;
end;





procedure TForm1.Inp_time_up;
{ Time button: RELEASE                                                         }
begin
  btn_time_down.Visible := false;

  if iMode = 3 then                                                             // Alarm ->
    begin
      iMode := 4; Gam_Set_Mode;                                                   //     -> Time
    end;
end;





procedure TForm1.Inp_acl_down;
{ ACL button: PRESS                                                            }
begin
  btn_acl_down.Visible := true and im_background_open.Visible;

  if iMode = 0 then                                                             // closed handheld game ->
    im_background_closedClick(Self)                                               //                    -> open handheld game
  else                                                                          // open handheld game ->
    begin
      iMode := 1; Gam_Set_Mode;                                                   //                  -> ACL/1
    end;
end;





procedure TForm1.Inp_acl_up;
{ ACL button: RELEASE                                                          }
begin
  btn_acl_down.Visible := false;

  if iMode = 1 then                                                             // ACL/1 ->
    begin
      iMode := 2; Gam_Set_Mode;                                                   //     -> ACL/2
    end;
end;





procedure TForm1.Inp_right_down;
{ cross button (right): PRESS                                                  }
begin
  if im_background_open.Visible then
    begin
      btn_cross_down.Visible := true;
      btn_cross_down.Picture := btn_cross_down_right.Picture;
    end;

  if not bArrowKeyPressed then
    begin
      bArrowKeyPressed := true;
      iLoopMarioMove := 1;

      if bController then
        begin
          Timer_Mario_Move.Interval := TIMER_MARIO_DELAY_SHORT;
          Timer_Mario_Move.Enabled := true;
        end;
    end;
end;





procedure TForm1.Inp_right_up;
{ cross button (right): RELEASE                                                }
begin
  btn_cross_down.Visible := false;
  bArrowKeyPressed := false;
  iLoopMarioMove := 0;
end;





procedure TForm1.Inp_down_down;
{ cross button (down): PRESS                                                   }
begin
  if im_background_open.Visible then
    begin
      btn_cross_down.Visible := true;
      btn_cross_down.Picture := btn_cross_down_down.Picture;
    end;

  if not bArrowKeyPressed then
    begin
      bArrowKeyPressed := true;
      iLoopMarioMove := 2;

      if bController then
        begin
          Timer_Mario_Move.Interval := TIMER_MARIO_DELAY_SHORT;
          Timer_Mario_Move.Enabled := true;
        end;
    end;
end;





procedure TForm1.Inp_down_up;
{ cross button (down): RELEASE                                                 }
begin
  btn_cross_down.Visible := false;
  bArrowKeyPressed := false;
  iLoopMarioMove := 0;
end;





procedure TForm1.Inp_left_down;
{ cross button (left): PRESS                                                   }
begin
  if im_background_open.Visible then
    begin
      btn_cross_down.Visible := true;
      btn_cross_down.Picture := btn_cross_down_left.Picture;
    end;

  if not bArrowKeyPressed then
    begin
      bArrowKeyPressed := true;
      iLoopMarioMove := 3;

      if bController then
        begin
          Timer_Mario_Move.Interval := TIMER_MARIO_DELAY_SHORT;
          Timer_Mario_Move.Enabled := true;
        end;
    end;
end;





procedure TForm1.Inp_left_up;
{ cross button (left): RELEASE                                                 }
begin
  btn_cross_down.Visible := false;
  bArrowKeyPressed := false;
  iLoopMarioMove := 0;

  if (iMarioPosX = 1) and (iMarioPosY = 7) and bController then
    begin
      iMario0107Hand := 1;
      Vid_Show_Mario0107Hand(iMario0107Hand);
    end;
end;





procedure TForm1.Inp_up_down;
{ cross button (up): PRESS                                                     }
begin
  if im_background_open.Visible then
    begin
      btn_cross_down.Visible := true;
      btn_cross_down.Picture := btn_cross_down_up.Picture;
    end;

  if not bArrowKeyPressed then
    begin
      bArrowKeyPressed := true;
      iLoopMarioMove := 4;

      if bController then
        begin
          Timer_Mario_Move.Interval := TIMER_MARIO_DELAY_SHORT;
          Timer_Mario_Move.Enabled := true;
        end;
    end;
end;





procedure TForm1.Inp_up_up;
{ cross button (up): RELEASE                                                   }
begin
  btn_cross_down.Visible := false;
  bArrowKeyPressed := false;
  iLoopMarioMove := 0;
end;





procedure TForm1.Inp_jump_down;
{ jump button: PRESS                                                           }
begin
  btn_jump_down.Visible := true and im_background_open.Visible;

  if not bJumpKeyPressed then                                                   // if jump key is not being pressed
    begin
      bJumpKeyPressed := true;                                                    // jump key has now been pressed

      if bController then                                                         // if ingame
        begin
          if (iMarioPosX = 0) and (iMarioPosY = 0) then                             // if Mario in (0,0)
            begin
              Gam_Mario_Jumps_In;                                                     // Mario jumps to (1,1)
              Dec(iLives); Vid_Show_Panel_Lives(iLives);                              // lives decrease by 1
            end
          else
            if ((iMarioPosX = 1) and (iMarioPosY = 1)) or                               // if Mario in (1,1)
               ((iMarioPosX = 4) and (iMarioPosY = 1)) or                               // or Mario in (4,1)
               ((iMarioPosX = 3) and (iMarioPosY = 3)) or                               // or Mario in (3,3)
               ((iMarioPosX = 4) and (iMarioPosY = 3)) then                             // or Mario in (4,3)
              begin
                bController := false;

                Vid_Show_Mario(iMarioPosX, iMarioPosY, false);
                Inc(iMarioPosY);
                Vid_Show_Mario(iMarioPosX, iMarioPosY, true);

                Snd_Play(pfs_jump);

                // collision check with iron
                if (aIm_iron[3].Visible and aIm_mario[3,4].Visible)
                  or (aIm_iron[4].Visible and aIm_mario[4,4].Visible) then
                  begin
                    Gam_Stop_All_Timers;
                    bController := false;

                    Vid_Show_Panel_Score(iScore, true);                                   // forces score display in case it was blinking
                    iScoreMultiplier := 1;                                                // resets score multiplier to 1

                    iLoopMiss := 1;
                    Timer_Miss.Interval := 1;
                    Timer_Miss.Enabled := true;
                  end;

              end
            else
              if (iMarioPosX = 5) and (iMarioPosY = 7) then                             // if Mario in (5,7)
                begin
                  bController := false;

                  // all timers stop except TimerBlink (in case it's active)
                  Timer_Game.Enabled := false;
                  Timer_Hook.Enabled := false;
                  Timer_Mario_Move.Enabled := false;

                  Vid_Show_Mario(iMarioPosX, iMarioPosY, false);
                  aIm_mario_06_07[1].Visible := true;

                  Snd_Play(pfs_jump);

                  if aIm_hook[1].Visible then                                             // if hook is on the leftmost side
                    begin                                                                   // jump is ok!
                      iLoopBonus := 1;                                                        // bonus routine
                      Timer_Bonus.Interval := 1;
                      Timer_Bonus.Enabled := true;
                    end
                  else
                    begin                                                                   // miss occurs
                      Timer_Score_Blink.Enabled := false;                                     // also TimerScoreBlink stops (in case it's active)

                      Vid_Show_Panel_Score(iScore, true);                                     // forces score display in case it was blinking

                      iLoopMiss := 1;
                      Timer_Miss_Jump.Interval := 1;
                      Timer_Miss_Jump.Enabled := true;
                    end;
                end;
        end; // if bController
    end;
end;





procedure TForm1.Inp_jump_up;
{ jump button: RELEASE                                                         }
begin
  btn_jump_down.Visible := false;
  bJumpKeyPressed := false;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME ROUTINES                                                * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Gam_Stop_All_Timers;
{ stops all timers at once                                                     }
begin
  Timer_Bonus.Enabled          := false;
  Timer_Bonus_Chance.Enabled   := false;
  Timer_Bonus_Complete.Enabled := false;
  Timer_Clock.Enabled          := false;
  Timer_Game.Enabled           := false;
  Timer_Game_Over.Enabled      := false;
  Timer_Hook.Enabled           := false;
  Timer_Mario_Move.Enabled     := false;
  Timer_Miss.Enabled           := false;
  Timer_Miss_Jump.Enabled      := false;
  Timer_Score_Blink.Enabled    := false;
end;





procedure TForm1.Gam_Mario_Jumps_In;
{ Mario jumps out from the "pit" and goes into (1,1); no lives are decreased   }
begin
  iMarioPosX := 1; iMarioPosY := 1; Vid_Show_Mario(iMarioPosX, iMarioPosY, true);
  Snd_Play(pfs_jump);

  Timer_Mario_Move.Interval := TIMER_MARIO_DELAY_LONG;
  Timer_Mario_Move.Enabled := bArrowKeyPressed;
end;





procedure TForm1.Gam_Set_Game_Start;
{ inits all game variables                                                     }
  var
    i, j: integer;
begin
  for i := 1 to 6 do
    for j := 1 to 3 do
      abarrel[i, j * 2 - 1] := 0;

  for i := 1 to 3 do
    for j := 6 to 8 do abarrel[i * 2, j] := 0;

  abarrel[6, 1] := 1; aIm_barrel[6, 1].Visible := true;
  abarrel[4, 3] := 1; aIm_barrel[4, 3].Visible := true;
  abarrel[4, 6] := 1; aIm_barrel[4, 6].Visible := true;

  aIm_girder[1].Visible := true;
  aIm_lever[1].Visible  := true;

  iLoopHook := 0;

  iScoreMultiplier := 1;

  iScoreChanceNext := 300;

  aIm_crane[1].Visible := true;

  iWires := 4; Vid_Show_Wires(iWires);

  iKongPosX := 6; iKongPosY := 3; Vid_Show_Kong(iKongPosX, iKongPosY);

  iMarioPosX := 0; iMarioPosY := 0;
  iMario0107Hand := 0;
end;





procedure TForm1.Gam_Set_Mode;
{ sets game/time/ACL mode, depending on the value of the iMode variable        }
begin
  case iMode of
    0: begin                                                                    // handheld is closed
       end;
    1: begin                                                                    // ACL/1 (ACL button press)
         Gam_Stop_All_Timers;
         Snd_Silence;

         bController := false;
       end;
    2: Vid_Show_All(true);                                                      // ACL/2 (ACL button release)
    3: begin                                                                    // ALARM (Time button press)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);
         im_panel_time_colon.Visible := true;
         im_panel_time_pm.Visible := true;
         aIm_panel_digit_pos[1].Picture := aIm_panel_number[1].Picture;
         aIm_panel_digit_pos[2].Picture := aIm_panel_number[2].Picture;
         aIm_panel_digit_pos[3].Picture := aIm_panel_number[0].Picture;
         aIm_panel_digit_pos[4].Picture := aIm_panel_number[0].Picture;

         bController := false;
       end;
    4: begin                                                                    // TIME (Time button release)
         Gam_Stop_All_Timers;

         im_panel_time_colon.Visible := true;

         Timer_Clock.Interval := 1;
         Timer_Clock.Enabled := true;

         bController := false;
       end;
    5, 7: begin                                                                 // HISCORE A/B (Game A/B button press)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);

         case iMode of
           5: begin
                im_panel_game_a.Visible := true; Vid_Show_Panel_Score(iHiScoreA, true);
              end;
           7: begin
                im_panel_game_b.Visible := true; Vid_Show_Panel_Score(iHiScoreB, true);
              end;
         end; // case..of

         iLives := 3; Vid_Show_Panel_Lives(iLives);

         Gam_Set_Game_Start;

         bController := false;
       end;
    6, 8: begin                                                                 // GAME A/B (Game A/B button release)
         iScore := 0; Vid_Show_Panel_Score(iScore, true);

         iLoopGame := 1;
         iCounterGame := 1;
         Gam_Set_TimerGame_Interval;
         Timer_Game.Enabled := true;

         bController := true;
       end;
    9: begin                                                                    // GAME OVER
         // do nothing
       end;
  end; // case..of
end;





procedure TForm1.Gam_Set_TimerGame_Interval;
{ calculates TimerGame.Interval depending on iScore and iCounterGame           }
  var
    i: integer;
begin
  case iMode of
    6: i := 500 - (iCounterGame)             - (iScore div 3);                  // Game A
    8: i := 500 - (iCounterGame * 13 div 10) - (iScore div 3);                  // Game B
  end; // case..of

  if i < 100 then
    i := 100;

  Timer_Game.Interval := i;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS                                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_BonusTimer(Sender: TObject);
{ occus when 1 wire has been cut                                               }
begin
  case iLoopBonus of
    1: Timer_Bonus.Interval := 400;
    2: begin
         iLoopHook := 0;                                                        // stops hook now!

         aIm_mario_06_07[1].Visible := false;
         aIm_mario_06_07[2].Visible := true;
         aIm_hook[1].Visible :=        false;
         aIm_hook[3].Visible :=        true;

         Snd_Play(pfs_step);
       end;
    3: begin
         aIm_mario_06_07[2].Visible := false;
         aIm_hook[3].Visible :=        false;
         aIm_crane[2].Visible :=       false;
         aIm_crane[3].Visible :=       true;

         Snd_Play(pfs_step);

         Dec(iWires); Vid_Show_Wires(iWires);

         if iCounterGame < 11 then
           iCounterBonus := 19
         else
           iCounterBonus := 19 - iCounterGame div 16;

         if iCounterBonus < 5 then
           iCounterBonus := 5;

         Timer_Bonus.Interval := 95;
       end;
    4: if iCounterBonus > 0 then
         begin
           Dec(iCounterBonus);
           Inc(iScore, iScoreMultiplier);
           Vid_Show_Panel_Score(iScore, true);

           Snd_Play(pfs_bonus);

           iLoopBonus := 3;                                                     // prevents from becoming 5 after next Inc(iLoopBonus);

           // checks for "bonus score"
           if iScore >= iScoreChanceNext then
             begin
               Gam_Stop_All_Timers;
               Vid_Show_Panel_Score(iScore, true);                              // forces score display in case it was blinking
               Snd_Silence;
               bController := false;

               iBonusChanceReprise := 2;                                        // needed in TimerBonusChance

               iLoopBonusChance := 1;
               Timer_Bonus_Chance.Interval := 1;
               Timer_Bonus_Chance.Enabled := true;
             end;
         end
       else
         if iWires = 0 then
           begin
             iLoopBonusComplete := 1;
             Timer_Bonus_Complete.Interval := 1;
             Timer_Bonus_Complete.Enabled := true;

             Timer_Bonus.Enabled := false;
           end
         else
           Timer_Bonus.Interval := 500;
    5: begin
         aIm_mario_06_07[2].Visible := true;
         aIm_hook[3].Visible        := true;
         aIm_crane[3].Visible       := false;
         aIm_crane[2].Visible       := true;
       end;
    6: begin
         aIm_mario_06_07[2].Visible := false;
         aIm_mario_06_07[5].Visible := true;
         aIm_hook[3].Visible        := false;
         aIm_crane[2].Visible       := false;
         aIm_crane[1].Visible       := true;
         aIm_lever[2].Visible       := false;
         aIm_lever[1].Visible       := true;
       end;
    7: begin
         aIm_mario_06_07[5].Visible := false;
         Snd_Play(pfs_jump);
         Timer_Bonus.Interval := 1500;
       end;
    8: begin
         aBarrel[2, 1] := 0; aIm_barrel[2, 1].Visible := false;
         aBarrel[3, 1] := 0; aIm_barrel[3, 1].Visible := false;
         iMarioPosX := 1; iMarioPosY := 1;
         Vid_Show_Mario(iMarioPosX, iMarioPosY, true);

         Snd_Play(pfs_jump);

         iCounterGame := 1;
         Gam_Set_TimerGame_Interval;
         Timer_Game.Enabled := true;

         bController := true;

         Timer_Mario_Move.Interval := TIMER_MARIO_DELAY_LONG;
         Timer_Mario_Move.Enabled := bArrowKeyPressed;

         Timer_Bonus.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopBonus);
end;





procedure TForm1.Timer_Bonus_ChanceTimer(Sender: TObject);
{ occurs when 300 points score are reached                                     }
  var
    i: integer;
begin
  case iLoopBonusChance of
    1: Timer_Bonus_Chance.Interval := 500;
    2: begin
         Inc(iLives);

         if iLives > 3 then
           iLives := 3;

         Vid_Show_Panel_Lives(iLives); Snd_Play(pfs_bonus);
       end;
    3: begin i := iLives - 1; Vid_Show_Panel_Lives(i);                           end;
    4: begin                  Vid_Show_Panel_Lives(iLives); Snd_Play(pfs_bonus); end;
    5: begin i := iLives - 1; Vid_Show_Panel_Lives(i);                           end;
    6: begin                  Vid_Show_Panel_Lives(iLives); Snd_Play(pfs_bonus); end;
    7: begin
         if iLives >= 3 then
           begin
             iScoreMultiplier := 2;

             iLoopScoreBlink := 1;
             Timer_Score_Blink.Interval := 500;
             Timer_Score_Blink.Enabled := true;
           end;

         // which timer reprises?
         case iBonusChanceReprise of
           1: begin
                Timer_Game.Enabled := true;

                bController := true;

                Timer_Mario_Move.Interval := TIMER_MARIO_DELAY_SHORT;
                Timer_Mario_Move.Enabled := bArrowKeyPressed;
              end;
           2: Timer_Bonus.Enabled := true;
           3: Timer_Bonus_Complete.Enabled := true;
         end; // case..of

         if iLoopHook <> 0 then                                                 // if hook was swinging before TimerBonusChance started
           Timer_Hook.Enabled := true;                                           // TimerHook reprises

         Inc(iScoreChanceNext, 1000);                                           // sets next bonus score (+1000 points)

         Timer_Bonus_Chance.Enabled := false;
       end;
  end; //case..of

  Inc(iLoopBonusChance);
end;





procedure TForm1.Timer_Bonus_CompleteTimer(Sender: TObject);
{ occurs when all wires have been cut                                          }
begin
  case iLoopBonusComplete of
    1: Timer_Bonus_Complete.Interval := 95;
    2: begin
         aIm_girder[1].Visible := false;

         Snd_Play(pfs_jump);

         Timer_Bonus_Complete.Interval := 500;
       end;
    3: aIm_girder[1].Visible   := true;
    4: begin
         aIm_girder[1].Visible := false;

         Snd_Play(pfs_jump);
       end;
    5: aIm_girder[1].Visible   := true;
    6: begin
         aIm_girder[1].Visible := false;

         Snd_Play(pfs_jump);
       end;
    7: aIm_girder[1].Visible   := true;
    8: begin
         aIm_girder[1].Visible := false;
         aIm_girder[2].Visible := true;

         iKongPosX := 0; iKongPosY := 0;
         Vid_Show_Kong(iKongPosX, iKongPosY);                                   // hides Doney Kong
         im_kong_7.Visible     := true;

         Snd_Play(pfs_kongfall);

         Timer_Bonus_Complete.Interval := 150;
       end;
    9: begin
         aIm_girder[2].Visible := false;

         iCounterBonusComplete := 0;

         Timer_Bonus_Complete.Interval := 95;
       end;
   10: if iCounterBonusComplete < 20 then
         begin
           Inc(iCounterBonusComplete);
           Inc(iScore, iScoreMultiplier);
           Vid_Show_Panel_Score(iScore, true);

           if (iCounterBonusComplete = 1)
           or (iCounterBonusComplete = 7)
           or (iCounterBonusComplete = 13)
           or (iCounterBonusComplete = 19) then
             begin
               aIm_heart[2].Visible := false;
               aIm_heart[1].Visible := true;
             end
           else
             if (iCounterBonusComplete = 4)
             or (iCounterBonusComplete = 10)
             or (iCounterBonusComplete = 16) then
               begin
                 aIm_heart[2].Visible := true;
                 aIm_heart[1].Visible := false;
               end;

           Snd_Play(pfs_bonus);

           iLoopBonusComplete := 9;                                             // prevents from becoming 11 after next Inc(iLoopBonusComplete);

           // checks for "bonus score"
           if iScore >= iScoreChanceNext then
             begin
               Gam_Stop_All_Timers;
               Vid_Show_Panel_Score(iScore, true);                              // forces score display in case it was blinking
               Snd_Silence;
               bController := false;

               iBonusChanceReprise := 3;                                        // needed in TimerBonusChance

               iLoopBonusChance := 1;
               Timer_Bonus_Chance.Interval := 1;
               Timer_Bonus_Chance.Enabled := true;
             end;
         end
       else
         begin
           aIm_heart[1].Visible := false;
           aIm_heart[2].Visible := false;

           Timer_Bonus_Complete.Interval := 500;
         end;
   11: begin
         aIm_mario_06_07[2].Visible := true;
         aIm_hook[3].Visible        := true;
         aIm_crane[3].Visible       := false;
         aIm_crane[2].Visible       := true;
       end;

   12: begin
         aIm_mario_06_07[2].Visible := false;
         aIm_mario_06_07[5].Visible := true;
         aIm_hook[3].Visible        := false;
         aIm_crane[2].Visible       := false;
         aIm_crane[1].Visible       := true;
         aIm_lever[2].Visible       := false;
         aIm_lever[1].Visible       := true;
       end;
   13: begin
         aIm_mario_06_07[5].Visible := false;

         Snd_Play(pfs_jump);

         Timer_Bonus_Complete.Interval := 700;
       end;
   14: im_kong_7.Visible := false;
   15: begin
         aIm_girder[1].Visible := true;
         iWires := 4;
         Vid_Show_Wires(iWires);

         // sets Donkey Kong coordinates depending on current value of iLoopGame
         iKongPosX := 6;

         if iLoopGame = 4 then
           iKongPosY := 3
         else
           iKongPosY := 2;

         Vid_Show_Kong(iKongPosX, iKongPosY);
         aBarrel[2, 1] := 0; aIm_barrel[2, 1].Visible := false;
         aBarrel[3, 1] := 0; aIm_barrel[3, 1].Visible := false;
         iMarioPosX := 1; iMarioPosY := 1;
         Vid_Show_Mario(iMarioPosX, iMarioPosY, true);

         Snd_Play(pfs_jump);

         Gam_Set_TimerGame_Interval;
         iCounterGame := 1;
         Timer_Game.Enabled := true;

         bController := true;

         Timer_Mario_Move.Interval := TIMER_MARIO_DELAY_LONG;
         Timer_Mario_Move.Enabled := bArrowKeyPressed;

         Timer_Bonus_Complete.Enabled := false;
       end;
  end; //case..of

  Inc(iLoopBonusComplete);
end;





procedure TForm1.Timer_ClockTimer(Sender: TObject);
{ transforms current time into "sprites" form, then displays it                }
  var
    present:              TDateTime;
    hour, min, sec, msec: Word;
begin
  present := Now;                                                               // gets system time...
  DecodeTime(present, hour, min, sec, msec);                                    // and decodes it

  if hour > 11 then                                                             // changes mode from 0/24 to 12 AM/12 PM
    begin
      Dec(hour, 12);
      im_panel_time_am.Visible := false;
      im_panel_time_pm.Visible := true;
    end
  else
    begin
      im_panel_time_am.Visible := true;
      im_panel_time_pm.Visible := false;
    end;

  if hour = 0 then                                                              // changes mode from 0/24 to 12 AM/12 PM
    hour := 12;

  if hour > 9 then                                                              // hour: tens
    aIm_panel_digit_pos[1].Picture := aIm_panel_number[hour div 10].Picture
  else
    aIm_panel_digit_pos[1].Picture := nil;

  aIm_panel_digit_pos[2].Picture := aIm_panel_number[hour mod 10].Picture;      // hour: units
  aIm_panel_digit_pos[3].Picture := aIm_panel_number[min  div 10].Picture;      // minute: tens
  aIm_panel_digit_pos[4].Picture := aIm_panel_number[min  mod 10].Picture;      // minute: units

  Timer_Clock.Interval := 1000;
end;





procedure TForm1.Timer_GameTimer(Sender: TObject);
{ main game action                                                             }
  var
    i, j,
    iBarrelCounter: integer;
    bMissBarrel: boolean;
begin
  // sets "has any miss occured?" to false now
  bMissBarrel := false;

  if (iMarioPosX = 0) and (iMarioPosY = 0) and (iCounterGame = 15) then         // if Mario in (0,0) and 15 ticks occured
    begin
      Gam_Mario_Jumps_In;                                                         // Mario jumps to (1,1)
      Dec(iLives); Vid_Show_Panel_Lives(iLives);                                  // lives decrease by 1
    end;

  if Odd(iLoopGame) then                                                        // odd ticks
    case iLoopGame of
      1: begin                                                                  // tick = 1
           // Donkey Kong moves/stands still
           if (iCounterGame = 1) and (iScore = 0) then                            // works only for very 1st TimerGame tick
             begin
               iKongPosX := 0; iKongPosY := 0;                                      // prevents from moving Donkey Kong now or generating new barrels
             end
           else                                                                   // works from 5th TimerGame tick onwards
             begin

               if iKongPosX = 2 then i := Random(2)                                 // -1 or 0
                 else
                   if iKongPosX = 4 then i := Random(3) - 1                         // -1 or 0 or +1
                     else
                       if iKongPosX = 6 then i := Random(2) - 1;                    // 0 or 1

               i := i * 2;                                                          // doubles value
               Inc(iKongPosX, i);                                                   // updates Donkey Kong X coordinate
               Vid_Show_Kong(iKongPosX, iKongPosY);                                 // displays Donkey Kong
             end;

           // Irons move
           case iMode of                                                          // varies from 1 to 24
             6: begin                                                               // game A
                  if iMarioPosY = 4 then
                    iMaxRandomIron := 5
                  else
                    if iMarioPosY = 3 then
                      iMaxRandomIron := 10
                    else
                      iMaxRandomIron := 20;

                  Dec(iMaxRandomIron, iScore div 100);

                  if iMaxRandomIron < 5 then
                    iMaxRandomIron := 5;
                end;
             8: begin                                                               // game B
                  if iMarioPosY = 4 then
                    iMaxRandomIron := 3
                  else
                    if iMarioPosY = 3 then
                      iMaxRandomIron := 6
                    else
                      iMaxRandomIron := 12;

                  Dec(iMaxRandomIron, iScore div 75);

                  if iMaxRandomIron < 3 then
                    iMaxRandomIron := 3;
                end;
           end; // case..of

           for i := 1 to 4 do                                                     // irons move in 1-4
             aIm_iron[i].Visible := aIm_iron[i + 1].Visible;

           aIm_iron[5].Visible :=                                                 // generates new iron in 5
             (Random(iMaxRandomIron) = 0) and (not aIm_iron[4].Visible);

           // collision check with iron
           if (aIm_iron[1].Visible and aIm_mario[1,4].Visible)
             or (aIm_iron[3].Visible and aIm_mario[3,4].Visible)
             or (aIm_iron[4].Visible and aIm_mario[4,4].Visible) then
             begin
               Gam_Stop_All_Timers;
               bController := false;

               Vid_Show_Panel_Score(iScore, true);                                // forces score display in case it was blinking
               iScoreMultiplier := 1;                                             // resets score multiplier to 1

               bMissBarrel := true;

               iLoopMiss := 1;
               Timer_Miss.Interval := 1;
               Timer_Miss.Enabled := true;
             end;
         end;
      3: begin                                                                  // tick = 3
           // Donkey Kong throws barrel/keeps hands down
           if (iCounterGame = 3) and (iScore = 0) then                            // works only for the very 3rd TimerGame tick
             begin
               iKongPosX := 4; iKongPosY := 3;
               Vid_Show_Kong(iKongPosX, iKongPosY);
             end
           else                                                                     // works from 7th TimerGame tick onwards
             begin
               // initializes barrel generation

               // case 1: higher change of new barrel as Mario goes up the girders
               iMaxRandomBarrel := 9 - iMarioPosY;                                  // varies from 2 to 8

               // case 2: no barrels on the screen
               iBarrelCounter := 0;

               for i := 1 to 6 do
                 for j := 1 to 8 do
                   iBarrelCounter := iBarrelCounter + aBarrel[i, j];

               // Kong raises his arms (barrel generation at next timer tick)
               if (Random(iMaxRandomBarrel) = 0)
                 or (iBarrelCounter < 2) then
                 iKongPosY := 3
               else
                 iKongPosY := 2;

               Vid_Show_Kong(iKongPosX, iKongPosY);
             end
         end;
    end // case..of
  else                                                                          // tick = 2 or 4
    begin
      // checks for collisions with barrels left-right
      bMissBarrel := false;

      if ((iMarioPosY = 1) and (aBarrel[iMarioPosX + 1, 1] = 1))
      or ((iMarioPosY = 3) and (aBarrel[iMarioPosX, 3] = 1))
      or ((iMarioPosY = 5) and (aBarrel[2, 5] = 1)) then
        bMissBarrel := true
      else                                                                        // else move barrels
        begin
          // barrels move (on screen barrels)
          for i := 1 to 5     do aBarrel[i, 1]     := aBarrel[i + 1, 1];
                                 aBarrel[6, 1]     := aBarrel[6, 3];
          for i := 6 downto 2 do aBarrel[i, 3]     := aBarrel[i - 1, 3];
          for i := 3 to 4     do aBarrel[1, i]     := aBarrel[1, i + 1];
          for i := 1 to 5     do aBarrel[i, 5]     := aBarrel[i + 1, 5];
                                 aBarrel[2, 5]     := aBarrel[2, 6] or aBarrel[2, 5];
                                 aBarrel[4, 5]     := aBarrel[4, 6] or aBarrel[4, 5];
                                 aBarrel[6, 5]     := aBarrel[6, 6];
          for i := 1 to 3 do
            for j := 6 to 7 do   aBarrel[i * 2, j] := aBarrel[i * 2, j + 1];
          for i := 1 to 3 do     aBarrel[i * 2, 8] := 0;

          if (iMarioPosY = 7) and (aBarrel[iMarioPosX + 1, 7] = 1) then             // checks collision with barrels from above
            bMissBarrel := true;
        end;

      if bMissBarrel then                                                         // a miss has occured

        begin
          Gam_Stop_All_Timers;
          bController := false;

          Vid_Show_Panel_Score(iScore, true);                                       // forces score display in case it was blinking
          iScoreMultiplier := 1;                                                    // resets score multiplier to 1

          iLoopMiss := 1;
          Timer_Miss.Interval := 1;
          Timer_Miss.Enabled := true;
        end

      else                                                                        // else proceed...

        begin
          // barrels move (generate new barrels + display Kong hands up/down)
          for i := 1 to 3 do                                                          // first set all new barrels to 0
            aBarrel[i * 2, 8] := 0;

          if (iCounterGame = 2) and (iScore = 0) then                                   // works for the very 2nd TimerGame tick only
            begin
              iKongPosX := 4; iKongPosY := 3;
              Vid_Show_Kong(iKongPosX, iKongPosY);
            end
          else
            if iKongPosY = 3 then
              begin                                                                     // eventually generates new barrel where Donkey Kong stands
                iKongPosY := 2;
                aBarrel[iKongPosX, 8] := 1;
                Vid_Show_Kong(iKongPosX, iKongPosY);
              end;

          // plays sound
          Snd_Play(pfs_barrel);

          // Mario lands after jump
          if (((iMarioPosX = 1) or (iMarioPosX = 4)) and (iMarioPosY = 2))            // if Mario is in (1,2) or (4,2)
          or (((iMarioPosX = 3) or (iMarioPosX = 4)) and (iMarioPosY = 4)) then       // or (4,3) or (4,4)
            begin                                                                       // Mario lands
              Vid_Show_Mario(iMarioPosX, iMarioPosY, false);
              Dec(iMarioPosY);
              Vid_Show_Mario(iMarioPosX, iMarioPosY, true);

              bController := true;

              Timer_Mario_Move.Interval := TIMER_MARIO_DELAY_SHORT;
              Timer_Mario_Move.Enabled := bArrowKeyPressed;

              // eventually adds score if jumped a barrel, and checks for "bonus score"
              if ((((iMarioPosX = 1) or (iMarioPosX = 4)) and (iMarioPosY = 1)) and (aBarrel[iMarioPosX, 1] = 1))
              or ((((iMarioPosX = 3) or (iMarioPosX = 4)) and (iMarioPosY = 3)) and (aBarrel[iMarioPosX + 1, 3] = 1)) then
                begin
                  Inc(iScore, iScoreMultiplier * (1 + (iMarioPosY div 2)) );            // score = "chance time" multiplier x1 (bottom girder) or x2 (middle girder)
                  Vid_Show_Panel_Score(iScore, true);

                  if iScore >= iScoreChanceNext then
                    begin
                      Gam_Stop_All_Timers;
                      Vid_Show_Panel_Score(iScore, true);                           // forces score display in case it was blinking
                      Snd_Silence;
                      bController := false;

                      iBonusChanceReprise := 1;                                     // needed in TimerBonusChance

                      iLoopBonusChance := 1;
                      Timer_Bonus_Chance.Interval := 1;
                      Timer_Bonus_Chance.Enabled := true;
                    end;
                end;

            end;

        end;

      // displays barrels (either value of bMissBarrel)
      for i := 1 to 3 do for j := 6 to 8 do aIm_barrel[i * 2, j].Visible     := aBarrel[i * 2, j] = 1;
      for i := 1 to 6 do for j := 1 to 3 do aIm_barrel[i, j * 2 - 1].Visible := aBarrel[i, j * 2 - 1] = 1;
      aIm_barrel[1, 4].Visible := (aBarrel[1, 4] = 1);
    end;

  // calculates speed for next TimerGame tick
  Gam_Set_TimerGame_Interval;

  // increases iCounterGame if no misses occured
  if not bMissBarrel then
    Inc(iCounterGame);

  // loops timer action (either value of bMissBarrel)
  Inc(iLoopGame);

  if iLoopGame > 4 then
    iLoopGame := 1;
end;





procedure TForm1.Timer_Game_OverTimer(Sender: TObject);
{ no lives left. plays "game over" tune, updates highscores and goes into      }
{ "time mode" after 5 minutes                                                  }
begin
  case iLoopGameOver of
    1: Timer_Game_Over.interval := 200;
    2: begin
         Vid_Show_All_Mario(false);

         Snd_Play(pfs_gameover);

         Timer_Game_Over.Interval := 500;
       end;
    3: begin
         if (iMode = 6) then
           begin
             if (iScore > iHiScoreA) and (iScore < 1000) then                   // if current score < 1000 and higher than current hiscore
               iHiScoreA := iScore;                                               // sets hiscore = current score

             if iScore > 999 then                                               // if current score > 999
               iHiScoreA := 998;                                                  // sets hiscore = 998

             iReg.WriteInteger('', 'HiScore1', iHiScoreA);
           end;

         if (iMode = 8) then
           begin
             if (iScore > iHiScoreB) and (iScore < 1000) then                   // if current score < 1000 and higher than current hiscore
               iHiScoreB := iScore;                                               // sets hiscore = current score

             if iScore > 999 then                                               // if current score > 999
               iHiScoreB := 998;                                                  // sets hiscore = 998

             iReg.WriteInteger('', 'HiScore2', iHiScoreB);
           end;

         iMode := 9; Gam_Set_Mode;

         Timer_Game_Over.Interval := 300000;
       end;
    4: begin                                                                    // goes automatically to time mode after 5 minutes
         Vid_Show_All(false);

         iMode := 4; Gam_Set_Mode;

         Timer_Game_Over.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopGameOver);
end;





procedure TForm1.Timer_HookTimer(Sender: TObject);
{ hook animation, works when lever is set up                                   }
begin
  case iLoopHook of
    1: begin                                                                    // lasts twice the time than other ticks
         aIm_lever[1].Visible := false;
         aIm_lever[2].Visible := true;
         aIm_crane[1].Visible := false;
         aIm_crane[2].Visible := true;
         Vid_Show_Hook(1);

         Timer_Hook.Interval := 450;
       end;
    3: Vid_Show_Hook(2);
    4: Vid_Show_Hook(3);
    5: Vid_Show_Hook(4);
    6: Vid_Show_Hook(5);                                                        // lasts twice the time
    8: Vid_Show_Hook(4);
    9: Vid_Show_Hook(3);
    10: Vid_Show_Hook(2);
    11: Vid_Show_Hook(1);                                                       // lasts twice the time
    13: Vid_Show_Hook(2);
    14: Vid_Show_Hook(3);
    15: begin
          Vid_Show_Hook(0);
          aIm_lever[2].Visible := false;
          aIm_lever[1].Visible := true;
          aIm_crane[2].Visible := false;
          aIm_crane[1].Visible := true;

          Timer_Hook.Interval := 300;
        end;
    16: begin
          iLoopHook := -1;                                                      // becomes 0 with next "Inc()"

          Timer_Hook.Enabled := false;
        end;
  end; // case..of

  Inc(iLoopHook);
end;





procedure TForm1.Timer_Mario_MoveTimer(Sender: TObject);
{ manages Mario movements depending on iLoopMarioMove variable set before call }
var
  bMarioAt0107,
  bMissBarrel,
  bMoved:      boolean;
begin
  bMarioAt0107 := false;
  bMissBarrel  := false;
  bMoved       := false;

  case iLoopMarioMove of
    1: begin                                                                    // Mario moves right
         if (iMarioPosX < 5) and ((iMarioPosY = 1) or (iMarioPosY = 3)) then      // from (1-to-4,1) or (1-to-4,3)

           if aBarrel[iMarioPosX + 1, iMarioPosY] = 1 then                          // checks collision with barrels on the right
             bMissBarrel := true
           else
             begin                                                                  // or Mario moves
               Vid_Show_Mario(iMarioPosX, iMarioPosY, false);
               Inc(iMarioPosX);
               Vid_Show_Mario(iMarioPosX, iMarioPosY, true);
               bMoved := true;
             end

         else

           if (iMarioPosX < 5) and (iMarioPosY = 7) then                          // from (1-to-4,7)
             begin                                                                  // Mario moves
               if iMarioPosX = 1 then
                 begin
                   iMario0107Hand := 0;
                   Vid_Show_Mario0107Hand(iMario0107Hand);
                 end;

               Vid_Show_Mario(iMarioPosX, iMarioPosY, false);
               Inc(iMarioPosX, 2);
               Vid_Show_Mario(iMarioPosX, iMarioPosY, true);
               bMoved := true;

               if aBarrel[iMarioPosX + 1, iMarioPosY] = 1 then                      // checks collision with barrels from above
                 bMissBarrel := true;
             end

           else

             if (iMarioPosX = 5) and (iMarioPosY = 7) then                        // from (5,7)
               begin                                                                // miss occurs
                 Gam_Stop_All_Timers;
                 bController := false;
                 Vid_Show_Mario(iMarioPosX, iMarioPosY, false);

                 Vid_Show_Panel_Score(iScore, true);                            // forces score display in case it was blinking
                 iScoreMultiplier := 1;                                         // resets score multiplier to 1

                 iLoopMiss := 2;
                 Timer_Miss_Jump.Interval := 1;
                 Timer_Miss_Jump.Enabled := true;
               end;
       end;
    2: begin                                                                    // Mario moves down
         if (iMarioPosX = 5) and (iMarioPosY = 3) then                            // from (5,3)
           begin
             Vid_Show_Mario(iMarioPosX, iMarioPosY, false);
             Dec(iMarioPosY, 2);
             Vid_Show_Mario(iMarioPosX, iMarioPosY, true);
             bMoved := true;
           end
         else
           if (iMarioPosX = 1) and (iMarioPosY = 4) then                          // from (1,4)
             begin
               Vid_Show_Mario(iMarioPosX, iMarioPosY, false);
               Dec(iMarioPosY, 1);
               Vid_Show_Mario(iMarioPosX, iMarioPosY, true);
               bMoved := true;
             end
           else
             if (iMarioPosX = 1) and (iMarioPosY = 5) then                        // from (1,5)
               begin
                 Vid_Show_Mario(iMarioPosX, iMarioPosY, false);
                 Dec(iMarioPosY, 1);
                 Vid_Show_Mario(iMarioPosX, iMarioPosY, true);
                 bMoved := true;

                 // collision check with iron
                 if aIm_iron[1].Visible then
                   begin
                     Gam_Stop_All_Timers;
                     bController := false;

                     Vid_Show_Panel_Score(iScore, true);                            // forces score display in case it was blinking
                     iScoreMultiplier := 1;                                         // resets score multiplier to 1

                     iLoopMiss := 1;
                     Timer_Miss.Interval := 1;
                     Timer_Miss.Enabled := true;
                   end;
               end
             else
               if (iMarioPosX = 1) and ((iMarioPosY = 7)) then                    // from (1,7)
                 begin
                   Vid_Show_Mario(iMarioPosX, iMarioPosY, false);
                   iMario0107Hand := 0;
                   Vid_Show_Mario0107Hand(iMario0107Hand);
                   Dec(iMarioPosY, 2);
                   Vid_Show_Mario(iMarioPosX, iMarioPosY, true);
                   bMoved := true;

                   if aBarrel[iMarioPosX + 1, iMarioPosY + 1] = 1 then              // checks collision with barrels from bottom
                     bMissBarrel := true;
                 end;
       end;
    3: begin                                                                    // Mario moves left
         if (iMarioPosX > 1) and ((iMarioPosY = 1) or (iMarioPosY = 3)) then      // from (2-to-5,1) or (2-to-5,3)
           if aBarrel[iMarioPosX, iMarioPosY] = 1 then                              // checks collision with barrels on the left
             bMissBarrel := true
           else
             begin                                                                  // or Mario moves
               Vid_Show_Mario(iMarioPosX, iMarioPosY, false);
               Dec(iMarioPosX);
               Vid_Show_Mario(iMarioPosX, iMarioPosY, true);
               bMoved := true;
             end
         else
           if (iMarioPosX > 1) and (iMarioPosY = 7) then                          // from (2-to-5,7)
             begin
               Vid_Show_Mario(iMarioPosX, iMarioPosY, false);
               Dec(iMarioPosX, 2);
               Vid_Show_Mario(iMarioPosX, iMarioPosY, true);
               bMoved := true;

               if iMarioPosX = 1 then
                 begin
                   iMario0107Hand := 1;
                   Vid_Show_Mario0107Hand(iMario0107Hand);
                 end;
             end
           else
             if (iMarioPosX = 1) and (iMarioPosY = 7) then                        // from (1,7) (hook)
               begin
                 iMario0107Hand := 2;
                 Vid_Show_Mario0107Hand(iMario0107Hand);

                 bMarioAt0107 := true;                                            // needed to reduce the controller's delay on next Mario-move

                 if (iLoopHook = 0) and (not Timer_Hook.Enabled) then
                   begin
                     iLoopHook := 1;
                     Timer_Hook.Interval := 1;
                     Timer_Hook.Enabled := true;
                   end;
               end
             else
               if (iMarioPosX = 1) and (iMarioPosY = 3) then                        // from (1,3)
                 if aBarrel[1, 3] = 1 then                                            // miss occurs if Mario attempts to go left from 1,3 onto a barrel
                   bMissBarrel := true;
       end;
    4: begin                                                                    // Mario moves up
         if (iMarioPosX = 5) and (iMarioPosY = 1) then                            // from (5,1)
           begin
             Vid_Show_Mario(iMarioPosX, iMarioPosY, false);
             Inc(iMarioPosY, 2);
             Vid_Show_Mario(iMarioPosX, iMarioPosY, true);
             bMoved := true;
           end
         else
           if (iMarioPosX = 1) and (iMarioPosY = 3) then                          // from (1,3)
             begin
               Vid_Show_Mario(iMarioPosX, iMarioPosY, false);
               Inc(iMarioPosY, 1);
               Vid_Show_Mario(iMarioPosX, iMarioPosY, true);
               bMoved := true;

               // collision check with iron
               if aIm_iron[1].Visible then
                 begin
                   Gam_Stop_All_Timers;
                   bController := false;

                   Vid_Show_Panel_Score(iScore, true);                              // forces score display in case it was blinking
                   iScoreMultiplier := 1;                                           // resets score multiplier to 1

                   iLoopMiss := 1;
                   Timer_Miss.Interval := 1;
                   Timer_Miss.Enabled := true;
                 end;
             end
           else
             if (iMarioPosX = 1) and (iMarioPosY = 4) then                        // from (1,4)
               begin
                 Vid_Show_Mario(iMarioPosX, iMarioPosY, false);
                 Inc(iMarioPosY, 1);
                 Vid_Show_Mario(iMarioPosX, iMarioPosY, true);
                 bMoved := true;
               end
             else
               if (iMarioPosX = 1) and ((iMarioPosY = 5)) then                      // from (1,5)
                 begin
                   Vid_Show_Mario(iMarioPosX, iMarioPosY, false);
                   Inc(iMarioPosY, 2);
                   Vid_Show_Mario(iMarioPosX, iMarioPosY, true);
                   iMario0107Hand := 1;
                   Vid_Show_Mario0107Hand(iMario0107Hand);
                   bMoved := true;

                   bMarioAt0107 := true;                                              // needed to reduce the controller's delay on next Mario-move

                   if aBarrel[iMarioPosX + 1, iMarioPosY] = 1 then                    // checks collision with barrels on the left
                     bMissBarrel := true
                 end;
       end;
  end; // case..of

  if bMissBarrel then                                                             // if miss occured
    begin                                                                           // executes TimerMiss procedure
      Gam_Stop_All_Timers;
      bController := false;

      Vid_Show_Panel_Score(iScore, true);                                           // forces score display in case it was blinking
      iScoreMultiplier := 1;                                                        // resets score multiplier to 1

      iLoopMiss := 1;
      Timer_Miss.Interval := 1;
      Timer_Miss.Enabled := true;
    end
  else                                                                            // else
    if bMoved then                                                                  // if Mario just moved
      Snd_Play(pfs_step);                                                             // sound sample plays

  // further movement if any keys are still pressed
  if bMarioAt0107 then                                                          // if Mario is next to the lever
    Timer_Mario_Move.Interval := TIMER_MARIO_DELAY_SHORT                          // forces short delay before next move
  else
    if bArrowKeyPressed then                                                      // else if any arrow key is still pressed
      Timer_Mario_Move.Interval := TIMER_MARIO_DELAY_LONG                           // long delay before next move
    else
      Timer_Mario_Move.Enabled := false;                                            // else stops walking
end;





procedure TForm1.Timer_MissTimer(Sender: TObject);
{ a miss has occured: Mario hit a barrel or an iron bar                        }
  var
    i: integer;
begin
  case iLoopMiss of
    1: begin
         Snd_Play(pfs_miss);
         Timer_Miss.Interval := 150;
       end;
    2: begin Timer_Miss.Interval := 230; Vid_Show_Mario(iMarioPosX, iMarioPosY, false); i := 0; Vid_Show_Mario0107Hand(i);              end;
    3: begin Timer_Miss.Interval := 210; Vid_Show_Mario(iMarioPosX, iMarioPosY, true);          Vid_Show_Mario0107Hand(iMario0107Hand); end;
    4: begin Timer_Miss.Interval := 230; Vid_Show_Mario(iMarioPosX, iMarioPosY, false); i := 0; Vid_Show_Mario0107Hand(i);              end;
    5: begin Timer_Miss.Interval := 210; Vid_Show_Mario(iMarioPosX, iMarioPosY, true);          Vid_Show_Mario0107Hand(iMario0107Hand); end;
    6: begin
         Vid_Show_Mario(iMarioPosX, iMarioPosY, false);
         iMario0107Hand := 0;
         Vid_Show_Mario0107Hand(iMario0107Hand);

         if iLives > 0 then
           begin
             iMarioPosX := 0;
             iMarioPosY := 0;

             iCounterGame := 1;
             Gam_Set_TimerGame_Interval;
             Timer_Game.Enabled := true;

             bController := true;

             Timer_Mario_Move.Interval := TIMER_MARIO_DELAY_SHORT;
             Timer_Mario_Move.Enabled := bArrowKeyPressed;

             if iLoopHook > 0 then
               Timer_Hook.Enabled := true;
           end
         else
           begin
             iLoopGameOver := 1;
             Timer_Game_Over.Interval := 1;
             Timer_Game_Over.Enabled := true;
           end;

         Timer_Miss.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopMiss);
end;





procedure TForm1.Timer_Miss_JumpTimer(Sender: TObject);
{ a miss has occured: Mario fell down from top girder                          }
begin
  case iLoopMiss of
    1: Timer_Miss_Jump.Interval := 400;
    2: begin
         Snd_Play(pfs_fall);
         aIm_mario_06_07[1].Visible := false;
         aIm_mario_06_07[3].Visible := true;

         Timer_Miss_Jump.Interval := 500;
       end;
    3: begin
         aIm_mario_06_07[3].Visible := false;
         aIm_mario_06_07[4].Visible := true;

         Timer_Miss_Jump.Interval := 800;
       end;
    4: begin
         if iLives > 0 then
           begin
             aIm_mario_06_07[4].Visible := false;
             iMarioPosX := 0;
             iMarioPosY := 0;

             iCounterGame := 1;
             Gam_Set_TimerGame_Interval;
             Timer_Game.Enabled := true;

             bController := true;

             Timer_Mario_Move.Interval := TIMER_MARIO_DELAY_SHORT;
             Timer_Mario_Move.Enabled := bArrowKeyPressed;

             if iLoopHook > 0 then
               Timer_Hook.Enabled := true;
           end
         else
           begin
             iLoopGameOver := 1;
             Timer_Game_Over.Interval := 1;
             Timer_Game_Over.Enabled := true;
           end;

         Timer_Miss_Jump.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopMiss);
end;





procedure TForm1.Timer_Score_BlinkTimer(Sender: TObject);
{ score panel blinks during "chance time"                                      }
begin
  Vid_Show_Panel_Score(iScore, Odd(iLoopScoreBlink));

  Inc(iLoopScoreBlink);

  if iLoopScoreBlink = 3 then
    iLoopScoreBlink := 1;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.pm_0_menuPopup(Sender: TObject);
{ shows the "checked" menuitems if needed. also, hides the game menu if it is  }
{ visible when the popup menu pops up                                          }
begin
{}pm_11_splashscreen.Checked := iReg.ReadBool('', 'SplashScreen', SplashForm.chb_splashscreen.Checked);
{}pm_12_controller.Checked := bDisplayCtrlOverlays;

{}if bFullscreenMode
    and bGameMenuOpen then
    begin
      bGameMenuOpen := false;
      Form1.Vid_Display_Game_Menu(bGameMenuOpen);
    end;
end;





procedure TForm1.pm_1_aboutClick(Sender: TObject);
{ displays the embedded "About the author..." form                             }
begin
  AboutForm.Show;
end;





procedure TForm1.pm_2_readmeClick(Sender: TObject);
{ opens the "Readme.txt" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Readme.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_3_whatsnewClick(Sender: TObject);
{ opens the "Whatsnew.txt" file                                                }
begin
  ShellExecute(Handle, 'open', 'Whatsnew.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_4_manualClick(Sender: TObject);
{ displays the embedded instructions manual                                    }
begin
  ManualForm.Show;
end;





procedure TForm1.pm_6_configClick(Sender: TObject);
{ opens the "Config.ini" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Config.ini', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_7_keysClick(Sender: TObject);
{ displays the "key settings" form                                             }
begin
  KeysForm.position := poScreenCenter;
  KeysForm.visible := true;
  Form1.Enabled := false;
end;





procedure TForm1.pm_8_hiscoreClick(Sender: TObject);
{ resets high scores to the system registry                                    }
begin
  iHiScoreA := 0; iReg.WriteInteger('', 'HiScore1', iHiScoreA);
  iHiScoreB := 0; iReg.WriteInteger('', 'HiScore2', iHiScoreB);
  Beep;
end;





{}procedure TForm1.pm_10_fullscreenClick(Sender: TObject);
{ changes from fullscreen to windowed, and vice-versa                          }
begin
  // updates popup menu
  bFullscreenMode := not bFullscreenMode;
  pm_10_fullscreen.Checked := bFullscreenMode;
  bGameMenuOpen := false;

  iReg.WriteBool('', 'FullscreenMode', bFullscreenMode);

  // graphics init
  Top  := - Screen.Height;
  Left := - Screen.Width;
  Gfx_Game_Init;
  Position := poScreenCenter;
end;





procedure TForm1.pm_11_splashscreenClick(Sender: TObject);
{ updates the ticks to the checkbox, the popup menu and the system registry    }
begin
{}pm_11_splashscreen.Checked := not pm_11_splashscreen.Checked;

{}iReg.WriteBool('', 'SplashScreen', pm_11_splashscreen.Checked);
end;





procedure TForm1.pm_12_controllerClick(Sender: TObject);
{ displays the "tranlucent" controller overlay on the directional cross        }
begin
{}pm_12_controller.Checked := not pm_12_controller.Checked;
{}bDisplayCtrlOverlays := pm_12_controller.Checked;

{}iReg.WriteBool('', 'CtrlrOverlay', pm_12_controller.Checked);

  Vid_Display_Ctrl_Overlay;
end;





procedure TForm1.pm_14_exitClick(Sender: TObject);
{ exits the game                                                               }
begin
  Application.Terminate;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.btn_game_menu_topClick(Sender: TObject);
{ sets the game menu as visible/hidden, then displays/hides it                 }
begin
  bGameMenuOpen := not bGameMenuOpen;
  Vid_Display_Game_Menu(bGameMenuOpen);
end;





procedure TForm1.la_menu_itemMouseEnter(Sender: TObject);
{ the game menu label bg colour becomes solid                                  }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := false;
end;





procedure TForm1.la_menu_itemMouseLeave(Sender: TObject);
{ the game menu label bg colour becomes transparent                            }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := true;
end;





procedure TForm1.la_menu_itemMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ hides the game menu, once a label has been pressed                           }
begin
  if Button = mbLeft then
    begin
      bGameMenuOpen := false;
      Vid_Display_Game_Menu(false);
    end;
end;





procedure TForm1.Vid_Display_Game_Menu(b: boolean);
{ displays/hides the game menu, depending on the value of bGameMenuOpen        }
  var
    i: integer;
begin
  case bGameMenuOpen of
    true: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i] := TLabel.Create(Form1);
            aMenuLabels[i].Parent := Form1;

            aMenuLabels[i].Top := 32 * i;
            aMenuLabels[i].Left := 0;

            aMenuLabels[i].AutoSize := false;
            aMenuLabels[i].Height := 32;
            aMenuLabels[i].Width := 256;
            aMenuLabels[i].Layout := tlCenter;

            aMenuLabels[i].Transparent := true;

            if aMenuItem[i].Caption <> '-' then
              begin
                aMenuLabels[i].Font.Name := 'Arial';
                aMenuLabels[i].Font.Color := clWhite;
                aMenuLabels[i].Font.Size := -12;
                aMenuLabels[i].Caption := '   ' + aMenuItem[i].Caption;

                if aMenuItem[i].Checked then
                  aMenuLabels[i].Caption := aMenuLabels[i].Caption + ' [V]';

                aMenuLabels[i].Color := $00000000;
                aMenuLabels[i].Cursor := crHandPoint;
                aMenuLabels[i].Enabled := aMenuItem[i].Enabled;

                if aMenuLabels[i].Enabled then
                  begin
                    aMenuLabels[i].OnClick      := aMenuItem[i].OnClick;
                    aMenuLabels[i].OnMouseEnter := la_menu_itemMouseEnter;
                    aMenuLabels[i].OnMouseLeave := la_menu_itemMouseLeave;
                    aMenuLabels[i].OnMouseUp    := la_menu_itemMouseUp;
                  end;
              end;
          end;

        btn_game_menu_background.Width := 256;
        btn_game_menu_background.Height := 32 * (MENU_ITEMS + 1)
      end;
    false: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i].Free;
          end;

        btn_game_menu_background.Height := 32;
        btn_game_menu_background.Width := 36;
      end;
    end; // case..of
end;





initialization
  bStartup := true;





end.
